using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.IO;

using MapApp.Model;
using MapApp.Ui.Controls;

namespace MapApp.Ui
{
    public partial class FrmMain : Form
    {
        FrmMarkers frmMarkers = null;
        FrmOptions frmOptions = null;
        FrmInfo frmInfo = null;
        FrmFind frmFind = null;
        FrmAbout frmAbout = null;
        public static FrmMain Instance;

        public void SetStatusText(string value)
        {
            statusLabel.Text = value;
        }

        public void SetCoordText(string value)
        {
            coordLabel.Text = value;
        }

        public FrmMain()
        {
            InitializeComponent();
            CustomInitialize();
        }

        private void CustomInitialize()
        {
            AppUtil.init(this);
            FrmMain.Instance = this;
            ////this.frmOptions = new FrmOptions();
            this.frmFind = new FrmFind();
            this.frmAbout = new FrmAbout();


            MapLocation initialCenterAndZoom = GoogleMapControl.CenterAndZoomFromString(Properties.Settings.Default.Map_CenterAndZoom);
            mapControl.InitialCenterAndZoom = initialCenterAndZoom;

            mapControl.MapMouseMove += new GoogleMapControl.OnMapMouseMove(googleMapControl1_MapMouseMove);
            mapControl.MapClick += new GoogleMapControl.OnMapClick(googleMapControl1_MapClick);
            //googleMapControl1.DocumentCompleted += new WebBrowserDocumentCompletedEventHandler(googleMapControl1_DocumentCompleted);

            StartPosition = FormStartPosition.Manual;
            Location = Properties.Settings.Default.FrmMain_Location;
            Size = Properties.Settings.Default.FrmMain_Size;
            WindowState = Properties.Settings.Default.FrmMain_WindowState;
            if (WindowState == FormWindowState.Minimized)
            {
                WindowState = FormWindowState.Normal; // never start a window minimized
            }

        }


        //void googleMapControl1_DocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
        //{
        //    MapLocation initialCenterAndZoom = GoogleMapControl.CenterAndZoomFromString(Properties.Settings.Default.Map_CenterAndZoom);
        //    MapControl.SetCenterAndZoom(initialCenterAndZoom);
        //}

        public GoogleMapControl MapControl
        {
            get { return mapControl; }
        }


        void googleMapControl1_MapClick(GoogleMapControl sender, MapClickEventArgs e)
        {
            //string msg = "Lat: " + e.MapLocation.Lat +
            //        ", Lng: " + e.MapLocation.Lng;
            //AppUi.StatusText = e.Button + " click: " + msg;
            if (e.Button == MouseButtons.Right)
            {
                sender.NewMarker(e.MapLocation, "Gumby Rules!!");
            }
        }

        void googleMapControl1_MapMouseMove(MapApp.Ui.Controls.GoogleMapControl sender, MapLocation loc)
        {
            string msg = String.Format("Lat: {0:n2}, Lng: {1:n2}", loc.Lat, loc.Lng);
            AppUi.CoordText = msg;
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Hide();
            this.Dispose();
        }



        private void optionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (frmOptions == null)
            {
                frmOptions = new FrmOptions();
            }
            frmOptions.Show();

        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmAbout.Show();
        }

        private void findToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmFind.Show();
        }

        private void FrmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.Hide();

            Properties.Settings.Default.Map_CenterAndZoom = GoogleMapControl.CenterAndZoomToString(MapControl.GetCenterAndZoom());

            if (WindowState == FormWindowState.Normal)
            {
                Properties.Settings.Default.FrmMain_Location = Location;
                Properties.Settings.Default.FrmMain_Size = Size;
            }
            else
            {
                Properties.Settings.Default.FrmMain_Location = RestoreBounds.Location;
                Properties.Settings.Default.FrmMain_Size = RestoreBounds.Size;
            }
            Properties.Settings.Default.FrmMain_WindowState = WindowState;
            Properties.Settings.Default.Save();

            if (frmAbout != null)
            {
                frmAbout.Hide();
                frmAbout.Dispose();
            }
            if (frmOptions != null)
            {
                frmOptions.Hide();
                frmOptions.Dispose();
            }
            if (frmInfo != null)
            {
                frmInfo.Hide();
                frmInfo.Dispose();
            }
            if (frmMarkers != null)
            {
                frmMarkers.Hide();
                frmMarkers.Dispose();
            }
            if (frmFind != null)
            {
                frmFind.Hide();
                frmFind.SaveHistoryFile();
                frmFind.Dispose();
            }
        }

        // TODO: BROKEN - doesn't do anything good.
        private void printPreviewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Bitmap bmp = new Bitmap(mapControl.Width, mapControl.Height);
            mapControl.DrawToBitmap(bmp, new Rectangle(mapControl.Location.X, mapControl.Location.Y,
                mapControl.Width, mapControl.Height));

            Form dummy = new Form();
            dummy.BackgroundImage = bmp;
            dummy.Show();
        }

        private void markersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (frmMarkers == null)
            {
                frmMarkers = new FrmMarkers();
            }
            frmMarkers.Show();
        }

        private void infoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (frmInfo == null)
            {
                frmInfo = new FrmInfo();
            }
            frmInfo.Show();
        }

    }
}